import pytest

from streetview import get_panorama
from streetview.download import iter_tile_info, iter_tiles


def test_that_all_the_tiles_are_generated():
    tiles = list(iter_tile_info(pano_id="z80QZ1_QgCbYwj7RrmlS0Q"))
    assert len(tiles) == 26 * 13


@pytest.mark.vcr()
def test_that_first_tile_can_be_saved():
    tiles = iter_tiles(pano_id="z80QZ1_QgCbYwj7RrmlS0Q")
    tile = next(tiles)
    tile.image.save("image.jpg", "jpeg")
    next(tiles)


@pytest.mark.skip(reason="Takes a really long time.")
def test_that_panorama_downloads_successfully():
    image = get_panorama(pano_id="z80QZ1_QgCbYwj7RrmlS0Q")
    image.save("image.jpg", "jpeg")



import os

import pytest

from streetview import get_streetview

GOOGLE_MAPS_API_KEY = os.environ.get("GOOGLE_MAPS_API_KEY", None)


@pytest.mark.vcr(filter_query_parameters=["key"])
def test_readme_metadata_example():
    from streetview import get_panorama_meta

    meta = get_panorama_meta(
        pano_id="_R1mwpMkiqa2p0zp48EBJg", api_key=GOOGLE_MAPS_API_KEY
    )
    print(meta)

    result = str(meta)
    expected = (
        "date='2019-08'"
        " location=Location(lat=41.89820659475458, lng=12.47644649615282)"
        " pano_id='_R1mwpMkiqa2p0zp48EBJg'"
    )

    assert result == expected


@pytest.mark.vcr(filter_query_parameters=["key"])
def test_get_streetview_returns_rbg_image():
    image = get_streetview(
        pano_id="_R1mwpMkiqa2p0zp48EBJg",
        api_key=GOOGLE_MAPS_API_KEY,
    )

    assert image.mode == "RGB"


@pytest.mark.vcr(filter_query_parameters=["key"])
def test_readme_basic_download_example():
    from streetview import get_streetview

    image = get_streetview(
        pano_id="z80QZ1_QgCbYwj7RrmlS0Q",
        api_key=GOOGLE_MAPS_API_KEY,
    )

    image.save("image.jpg", "jpeg")



import itertools
import time
from dataclasses import dataclass
from io import BytesIO
from typing import Generator

import requests
from PIL import Image


@dataclass
class TileInfo:
    x: int
    y: int
    fileurl: str


@dataclass
class Tile:
    x: int
    y: int
    image: Image.Image


def iter_tile_info(pano_id: str) -> Generator[TileInfo, None, None]:
    """
    Generate a list of a panorama's tiles and their position.
    """

    image_url = (
        "https://cbk0.google.com/cbk?output=tile&panoid={0:}&zoom=5&x={1:}&y={2:}"
    )

    for x, y in itertools.product(range(26), range(13)):
        yield TileInfo(
            x=x,
            y=y,
            fileurl=image_url.format(pano_id, x, y),
        )


def iter_tiles(pano_id: str) -> Generator[Tile, None, None]:
    for info in iter_tile_info(pano_id):
        # Try to download the image file
        while True:
            try:
                response = requests.get(info.fileurl, stream=True)
                break
            except requests.ConnectionError:
                print("Connection error. Trying again in 2 seconds.")
                time.sleep(2)

        image = Image.open(BytesIO(response.content))

        yield Tile(x=info.x, y=info.y, image=image)

        del response
        del image


def get_panorama(pano_id: str) -> Image.Image:
    """
    Downloads a streetview panorama.
    """

    tile_width = 512
    tile_height = 512

    panorama = Image.new("RGB", (26 * tile_width, 13 * tile_height))

    for tile in iter_tiles(pano_id=pano_id):
        panorama.paste(im=tile.image, box=(tile.x * tile_width, tile.y * tile_height))
        del tile

    return panorama



from setuptools import setup


def readme():
    with open("readme.md") as f:
        return f.read()


setup(
    name="streetview",
    version="0.0.0",
    description="Retrieve current and historical photos from Google Street View",
    long_description=readme(),
    long_description_content_type="text/markdown",
    url="https://github.com/robolyst/streetview",
    author="Adrian Letchford",
    author_email="me@dradrian.com",
    license="MIT",
    packages=["streetview"],
    zip_safe=False,
    install_requires=[
        "requests",
        "pillow",
        "pydantic",
    ],
)



import os

import pytest

from streetview import get_panorama_meta, search_panoramas

GOOGLE_MAPS_API_KEY = os.environ.get("GOOGLE_MAPS_API_KEY", None)


SYDNEY = {
    "lat": -33.8796052,
    "lon": 151.1655341,
}

BELGRAVIA = {
    "lat": 51.4986562,
    "lon": -0.1570917,
}

MIDDLE_OF_OCEAN = {
    "lat": 28.092432,
    "lon": -34.399243,
}

TUNIS = {
    "lat": 36.8032829,
    "lon": 10.1808486,
}


class GenericGetPanoidsTest:
    def setup_method(self):
        raise NotImplementedError()

    def test_that_there_is_at_least_one_item(self):
        assert len(self.result) >= 1

    def test_that_panoids_are_unique(self):
        panoids = [p.pano_id for p in self.result]
        uniques = list(dict.fromkeys(panoids))
        assert len(panoids) == len(uniques)

    @pytest.mark.vcr(filter_query_parameters=["key"])
    def test_that_dates_are_correct(self):
        ids = [p.pano_id for p in self.result if p.date is not None]
        dates = [p.date for p in self.result if p.date is not None]
        meta_dates = [
            get_panorama_meta(
                pano_id=pano_id,
                api_key=GOOGLE_MAPS_API_KEY,
            ).date
            for pano_id in ids
        ]

        assert dates == meta_dates


@pytest.mark.vcr
class TestPanoidsOnSydney(GenericGetPanoidsTest):
    def setup_method(self):
        self.result = search_panoramas(**SYDNEY)

    def test_that_there_are_the_expected_number_of_results(self):
        assert len(self.result) == 20


@pytest.mark.vcr
class TestPanoidsOnBelgravia(GenericGetPanoidsTest):
    def setup_method(self):
        self.result = search_panoramas(**BELGRAVIA)

    def test_that_there_are_the_expected_number_of_results(self):
        assert len(self.result) == 43


@pytest.mark.vcr
def test_readme_search_example():
    from streetview import search_panoramas

    panos = search_panoramas(lat=41.8982208, lon=12.4764804)
    first = panos[0]
    print(first)

    result = str(first)
    expected = (
        "pano_id='_R1mwpMkiqa2p0zp48EBJg'"
        " lat=41.89820676786453 lon=12.47644220919742"
        " heading=0.8815613985061646 pitch=89.001953125 roll=0.1744659692049026"
        " date='2019-08'"
    )

    assert result == expected


@pytest.mark.vcr
def test_search_where_there_are_no_results():
    result = search_panoramas(**MIDDLE_OF_OCEAN)
    assert len(result) == 0


@pytest.mark.vcr
def test_search_where_there_are_no_dates():
    result = search_panoramas(**TUNIS)

    dates = [p.date for p in result]
    assert dates == [None] * len(dates)
